﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using MessageWorkflow;
using MessageWorkflow.Message;
using MessageWorkflow.Handler;

namespace SimpleConsoleExamples.Examples
{
    public class TwoWayNumberCounterExample : IExample
    {
        Dictionary<string, int> _knownNumbers = new Dictionary<string, int>();
        WorkflowChannel _channel = new WorkflowChannel();

        private void GetAmountForWordFromCache(WorkflowRequestAction<string, int> wordAction)
        {
            if (!wordAction.IsCompleted)
            {
                if (_knownNumbers.ContainsKey(wordAction.Data))
                {
                    _knownNumbers[wordAction.Data]++;
                    wordAction.IsCompleted = true;

                    wordAction.SetResult(_knownNumbers[wordAction.Data]);
                }
            }
        }
        private void GetAmountForWordFromUserAsync(WorkflowRequestAction<string, int> wordAction, 
            Action callback, 
            Action<Exception> onError)
        {
            // Put on another thread, therefore similating a async call
            System.Threading.ThreadPool.QueueUserWorkItem(x =>
            {
                if (!wordAction.IsCompleted)
                {
                    Console.Write("This is a new entry! What amount should i start with? ");

                    int amount;
                    while (true)
                    {
                        string amountString = Console.ReadLine();
                        if (int.TryParse(amountString, out amount))
                            break;
                        else
                            Console.WriteLine("This is not a valid numer, try again!");
                    }

                    _knownNumbers[wordAction.Data] = amount;

                    wordAction.SetResult(_knownNumbers[wordAction.Data]);
                }

                callback();
            });
        }

        public void Run()
        {
            Console.WriteLine("\n\nEnter a word and watch how the right components in the workflow register the amount of times you entered that specific word, if the word is new, the workflow will ask you to specify a start count");

            _channel.Subscribe(new WorkflowHandlerConfiguration(WorkflowHandlerFactory.FromLambda<WorkflowRequestAction<string, int>>(GetAmountForWordFromCache))
            {
                Successors = new[] 
                {
                    new WorkflowHandlerConfiguration(WorkflowHandlerFactory.FromLambdaAsync<WorkflowRequestAction<string, int>>(GetAmountForWordFromUserAsync))                
                }
            });

            while (true)
            {
                Console.Write("Enter word: ");
                string word = Console.ReadLine().ToLower();

                if (word == "x")
                    break;

                int result = _channel.Dispatch(new WorkflowRequestAction<string, int>(word)).Result;

                Console.WriteLine("For word {0}, i got number: {1}", word, result);
            }
        }
    }
}
